------------------------ 
--
-- Broken Loot
--
-- Usage:
--
-- /bloot
--      display the UI (WORK IN PROGRESS)
-- /bloot broadcast
--      broadcasts all the loot to the raid

local enableTradeExtras = true;

local _, L = ...;

local function handler(msg)
  local command, rest = msg:match("^(%S*)%s*(.-)$");
  -- Any leading non-whitespace is captured into command;
  -- the rest (minus leading whitespace) is captured into rest.
  command = strlower(command)
  if command == "" then
    --/bloot 
    BLootMainDisplayFrame:Show();
    displayUsageAll();
  elseif command == "clear" or command == "c" then
    --/bloot clear
    actionClearList(rest)
  elseif command == "show" or command == "s" or command == "list" then
    --/bloot show
    actionBroadcastList("RAID");
  elseif command == "add" or command == "a" then
    --/bloot add <who> <item>
  	local lootData = {};
    lootData.who, lootData.item = rest:match("^(%S*)%s*(.-)$");
    if strlower(lootData.who) == "target" then
      lootData.who = UnitName("target")
    end
    lootData.id = generateLootId();
	  lootData.when = getServerDate(true); -- now!!!
    if isValidCharacter(lootData.who) and isValidItem(lootData.item) then
      actionAddLoot(lootData);
    else
      displayUsageAdd(true);
    end
  elseif command == "roll" or command =="r" then
    actionStartRolls();
  elseif command == "on" then
    if strlower(rest) == "" then
      BLootAddonSaved.Config.active = 1;
      sendMessage(format(L["ActiveEnabled"]))
    else
      sendMessage(format(L["ActiveUnknownArg"]))
    end
  elseif command == "off" then
    if strlower(rest) == "" then
      BLootAddonSaved.Config.active = 0;
      sendMessage(format(L["ActiveDisabled"]))
    else
      sendMessage(format(L["ActiveUnknownArg"]))
    end
  elseif command == "debug" then
    if strlower(rest) == "on" then
      BLootAddonSaved.Config.debugMode = 1;
      sendMessage(format(L["DebugEnabled"]))
    elseif strlower(rest) == "off" then
      BLootAddonSaved.Config.debugMode = 0;
      sendMessage(format(L["DebugDisabled"]))
    else
      sendMessage(format(L["DebugUnknownArg"]))
    end
  elseif command == "debugnospam" then
    if strlower(rest) == "on" then
      BLootAddonSaved.Config.debugMode_NoSpam = 1;
      sendMessage(format(L["DebugNoSpamEnabled"]))
    elseif strlower(rest) == "off" then
      BLootAddonSaved.Config.debugMode_NoSpam = 0;
      sendMessage(format(L["DebugNoSpamDisabled"]))
    else
      sendMessage(format(L["DebugNoSpamUnknownArg"]))
    end
  elseif command == "countdown" then
    -- need some validation on the value
    local countdown = tonumber(rest)
    if countdown == nil then
      sendMessage(format(L["CountDownUnknownArg"], rest))
    else
      if countdown >= 5 and countdown <= 30 then
        BLootAddonSaved.Config.Countdown = countdown;
        sendMessage(format(L["CountdownSetTo"], countdown))
      else
        sendMessage(format(L["CountdownBadArg"]))
      end
    end
  else
  -- If not handled above, display some sort of help message
    displayUsageAll(true);
  end
end

function displayUsageAll(header)
  if header == true then
    sendMessage(format(L["Syntax"]));
  end
  displayUsageClear(false);
  displayUsageAdd(false);
  displayUsageRoll(false);
  displayUsageShow(false);
end
function displayUsageClear(header)
  if header == true then
    sendMessage(format(L["Syntax"]));
  end
  sendMessage(format(L["SyntaxClear1"]));
  sendMessage(format(L["SyntaxClear2"]));
end
function displayUsageShow(header)
  if header == true then
    sendMessage(format(L["Syntax"]));
  end
  sendMessage(format(L["SyntaxShow1"]));
  sendMessage(format(L["SyntaxShow2"]));
end
function displayUsageAdd(header)
  if header == true then
    sendMessage(format(L["Syntax"]));
  end
  sendMessage(format(L["SyntaxAdd1"]));
  sendMessage(format(L["SyntaxAdd2"]));
  sendMessage(format(L["SyntaxAdd3"]));
  sendMessage(format(L["SyntaxAdd4"]));
end
function displayUsageRoll(header)
  if header == true then
    sendMessage(format(L["Syntax"]));
  end
  sendMessage(format(L["SyntaxRoll1"]));
  sendMessage(format(L["SyntaxRoll1"]));
end

---------
-- ACTIONS
function actionClearList()
  -- very simply just destroy the object for now...
  BLootAddonSaved.LootList = {};
  sendMessage(format(L["ClearListDone"]));
end
function actionBroadcastList(toWho)
  actionBroadcastListViaChat(toWho)
  actionBroadcastListViaAddon(toWho)
end
function actionBroadcastListViaChat(toWho)
  if BLootAddonSaved.LootList == nil then
    sendMessage(format(L["NoLootInList"]), "SELF");
  else
    local numItems = #(BLootAddonSaved.LootList) 
    if numItems > 0 then
      sendMessage("We have got "..numItems.." bits of loot to give away.", toWho); 
      for index, lootData in ipairs(BLootAddonSaved.LootList) do
        if not(lootData.item == nil) then
          sendMessage(lootData.item.." given by ".. lootData.who..".", toWho);
        end
      end
    else
    sendMessage(format(L["NoLootInList"]), "SELF");
    end
  end
end
function actionBroadcastListViaAddon()
  --sendMessage("actionBroadcastListViaAddon - NYI");
end
function actionAddLoot(lootData)
  if BLootAddonSaved.LootList == nil then
    BLootAddonSaved.LootList = {};
  end
  tinsert(BLootAddonSaved.LootList, lootData);
  actionAnounceItem(lootData)
end
function actionAnounceItem(lootData)
  actionAnounceViaChat(lootData);
  actionAnounceViaAddon(lootData);
end
function actionAnounceViaChat(lootData)
  sendMessage(format(L["LootGivenBy"], lootData.item, lootData.who), "RAID");
end
function actionAnounceViaAddon(lootData)
  --sendMessage("actionAnounceViaAddon - NYI");
end
function actionStartRolls()
  sendMessage("actionStartRolls()", "DEBUG");
  if BLootAddonSaved.LootList == nil then
    sendMessage(format(L["NoLootInListLeft"]), "SELF");
  else
    local numItems = #(BLootAddonSaved.LootList) 
    if numItems > 0 then
      sendMessage(format(L["LootCount"], numItems), "RAID");
      if BLootAddonSaved.State == nill then
        BLootAddonSaved.State = {}
      elseif BLootAddonSaved.State.RollInProgress == true and not(moreThanAMinuteAgo(BLootAddonSaved.State.RollStarted) ) then
        -- we have a roll already in progress so shouldn't try to start another one.
        sendMessage(format(L["RollsInProgress"]));
      else
        sendMessage("  actionStartRolls() - starting to give it all away", "DEBUG");
        BLootAddonSaved.State.LootToRollOn = 1;
        actionStartRollForItem();
      end
    else
      sendMessage(format(L["NoLootInListLeft"]), "SELF");
    end
  end
end

function actionStartRollForItem()
  sendMessage("actionStartRollForItem()", "DEBUG");
  local numItems = #(BLootAddonSaved.LootList)
  sendMessage("  actionStartRollForItem() numItems "..numItems..".", "DEBUG");
  if numItems > 0 then
    sendMessage("  actionStartRollForItem() BLootAddonSaved.State.LootToRollOn "..BLootAddonSaved.State.LootToRollOn..".", "DEBUG");
    if BLootAddonSaved.State.LootToRollOn <= numItems then
      sendMessage("  actionStartRollForItem() finding item.", "DEBUG");
      BLootAddonSaved.State.RollingOn = BLootAddonSaved.LootList[BLootAddonSaved.State.LootToRollOn]
      BLootAddonSaved.State.RollingOn.Rolls = {}
      BLootAddonSaved.State.RollingOn.Winner = nil
      BLootAddonSaved.State.RollingOn.Unwanted = false
      BLootAddonSaved.State.Countdown = BLootAddonSaved.Config.Countdown;
      BLootAddonSaved.State.RollInProgress = true;
      BLootAddonSaved.State.RollStarted = getServerDate(true)
      sendMessage(BLootAddonSaved.State.RollingOn.item, "DEBUG");
      sendMessage(format(L["StartingRollsFor"], BLootAddonSaved.State.RollingOn.item, BLootAddonSaved.State.RollingOn.who), "RAID");
      C_Timer.After(1, actioRollForItemTick) 
    else
      sendMessage(L["NoLootInListLeft"], "RAID");
    end
  end
end

function actioRollForItemTick()
  --sendMessage("actioRollForItemTick", "DEBUG");
  if BLootAddonSaved.State.RollInProgress then
    if BLootAddonSaved.State.Countdown == -1 then
      BLootAddonSaved.State.Countdown = BLootAddonSaved.State.Countdown - 1;
      C_Timer.After(1, actioRollForItemTick)
    elseif BLootAddonSaved.State.Countdown == -2 then
      actionRollForItemConfirmWinner()
    elseif BLootAddonSaved.State.Countdown == 0 then
      sendMessage("0 - Times up!", "RAID");
      BLootAddonSaved.State.Countdown = BLootAddonSaved.State.Countdown - 1;
      C_Timer.After(1, actioRollForItemTick)
    else
      if BLootAddonSaved.State.Countdown > 10 then
        if mod(BLootAddonSaved.State.Countdown, 10) == 0 then
          sendMessage(BLootAddonSaved.State.Countdown, "RAID");
        end
      elseif BLootAddonSaved.State.Countdown > 3 then
        if mod(BLootAddonSaved.State.Countdown, 5) == 0 then
          sendMessage(BLootAddonSaved.State.Countdown, "RAID");
        end
      else
        sendMessage(BLootAddonSaved.State.Countdown, "RAID");
      end
      BLootAddonSaved.State.Countdown = BLootAddonSaved.State.Countdown - 1;
      C_Timer.After(1, actioRollForItemTick)
    end
  end
end

function highestRollSort(a, b) return tonumber(a.roll) > tonumber(b.roll) end

function actionRollForItemConfirmWinner()
  local numRolls = #(BLootAddonSaved.State.RollingOn.Rolls);
  if numRolls==0 then
    sendMessage("No one wanted "..BLootAddonSaved.State.RollingOn.item..", guess it goes into the bin!", "RAID");
    BLootAddonSaved.State.RollingOn.Unwanted = true;
    actionStartTradeLoot();
  else
    sendMessage("We have "..numRolls.." people who have rolled on "..BLootAddonSaved.State.RollingOn.item..".", "RAID");
    sort(BLootAddonSaved.State.RollingOn.Rolls, highestRollSort)
    BLootAddonSaved.State.RollingOn.Winner = BLootAddonSaved.State.RollingOn.Rolls[1];   
    actionAnnounceWinner();
    actionStartTradeLoot();
  end
end

function actionAnnounceWinner()
  actionAnnounceWinnerChat();
  actionAnnounceWinnerAddon();
end

function actionAnnounceWinnerChat()
  sendMessage("The winner of  "..BLootAddonSaved.State.RollingOn.item.." is "..BLootAddonSaved.State.RollingOn.Winner.name.." with a roll of "..BLootAddonSaved.State.RollingOn.Winner.roll, "RAID");
end

function actionAnnounceWinnerAddon()
  --sendMessage("actionAnnounceWinnerAddon - NYI");
end

function actionStartTradeLoot()
  if enableTradeExtras then
    sendMessage("actionTradeLoot - starting", "DEBUG");
    if not(alreadyTrading()) then
      if #(BLootAddonSaved.LootList) > 0 then
        lootData = BLootAddonSaved.LootList[1]
        if lootData.Winner == nil then
          sendMessage("  actionTradeLoot - no one wanted!", "DEBUG");
          if lootData.Unwanted == true then
            sendMessage("No one wanted "..lootData.item.." so removing from list!");
            tremove(BLootAddonSaved.LootList, 1);
            sendMessage("  actionTradeLoot - removed that item from the list!", "DEBUG");
            actionStartRollForItem();
          else
            sendMessage("No winner for "..lootData.item.." but not unwanted?!");
            actionStartRollForItem();
          end
        else
          sendMessage("  actionTradeLoot - giving away "..lootData.item.." to "..lootData.Winner.name..".", "DEBUG");
          --ClearTarget();
          if lootData.Winner.name == UnitName("player") then
            sendMessage("You won "..lootData.item.."!");
            tremove(BLootAddonSaved.LootList, 1);
            sendMessage("  actionTradeLoot - removed that item from the list!", "DEBUG");
          else
            inRange = CheckInteractDistance(lootData.Winner.name, 2);
            if inRange then 
              InitiateTrade(lootData.Winner.name)
              currentTrade.startedByMe = true;
              currentTrade.tradeWith = lootData.Winner.name;
              currentTrade.wantToTradeWith = lootData.Winner.name;
            else
              sendMessage(lootData.Winner.name.." is too far away from "..UnitName("player").." to trade!", "RAID");
              actionStartRollForItem();
            end
          end
        end
      end
    end
  end
end

---------
-- HANDLERS
function handleRoll(name, roll, minRoll, maxRoll)
  if BLootAddonSaved.State.RollInProgress then
    -- Should we really care?
    if BLootAddonSaved.State.Countdown > 0 then
      local found = false;
      for index, roll in ipairs(BLootAddonSaved.State.RollingOn.Rolls) do
        if roll.name == name then
          found = true;
        end
      end
      if found == true then
        sendMessage(name.." is a dirty rotton cheat rolling more than once :)", "RAID")
      elseif minRoll == "1" and maxRoll == "100" then
        sendMessage(name.." rolled "..roll..".", "RAID")
        tinsert(BLootAddonSaved.State.RollingOn.Rolls, {name = name, roll = roll})
      else
        sendMessage(name.." is a cheat on rolls!!!!!"..minRoll.."/"..maxRoll, "RAID")
      end
    end
  end
end

function handleRegisterInterest(id, name, item)

end
      
function handleTradeComplete()
  local finishedTrade = copyTrade(currentTrade);
  currentTrade = resetTrade();
  if finishedTrade.cancel == 0 then  
    finishedTrade.openTrade = false;
    -- did we complete a trade?
    sendMessage("handleTradeComplete()", "DEBUG")
    local numTargetItems =  #(finishedTrade.targetItems)
    sendMessage("  handleTradeComplete() - numTargetItems"..numTargetItems, "DEBUG")
    if numTargetItems > 0 then
      -- recieved an item or two I think....?
        sendMessage("  handleTradeComplete() - Recieved "..numTargetItems.." items", "DEBUG")
        for i = 1, numTargetItems+1 do
          --sendMessage("  handleTradeComplete() - adding "..finishedTrade.targetItems[i]..".", "DEBUG")
          local lootData = {};
          lootData.who = finishedTrade.tradeWith; 
          lootData.item = finishedTrade.targetItems[i];
          lootData.id = generateLootId();
          lootData.when = getServerDate(true); -- now!!!
          actionAddLoot(lootData);
        end
    end
    local numPlayerItems =  #(finishedTrade.playerItems)
    sendMessage("  handleTradeComplete() - numTargetItems"..numPlayerItems, "DEBUG")
    if numPlayerItems > 0 then
      sendMessage("  handleTradeComplete() - Given away "..numPlayerItems.." items", "DEBUG")
      -- need to integrate the queue here to make sure we are giving items to who needs them
      for i = 1, numTargetItems+1 do
        sendMessage("  handleTradeComplete() - given away "..finishedTrade.playerItems[i].." to "..finishedTrade.tradeWith..".", "DEBUG")
        for lootIndex, lootData in ipairs(BLootAddonSaved.LootList) do
          if lootData.item == finishedTrade.playerItems[i] then
            tremove(BLootAddonSaved.LootList, lootIndex)
            sendMessage("  handleTradeComplete() - have given away "..lootData.item..".", "DEBUG")
          else
            sendMessage("  handleTradeComplete() - "..finishedTrade.playerItems[i].." is NOT the same as "..lootData.item..".", "DEBUG")
          end
        end
      end
      if #(BLootAddonSaved.LootList)>0 then
        actionStartRollForItem();
      end
    end
  else
    sendMessage("  handleTradeComplete() - cancelled trade?", "DEBUG")
  end
end

function handleTradeStarted()
  currentTrade.tradeWith = UnitName("NPC")
  if currentTrade.startedByMe == true then
    sendMessage("  TRADE_SHOW() I started it (I think)", "DEBUG")
    if #(BLootAddonSaved.LootList) > 0 then
      lootData = BLootAddonSaved.LootList[1]
      if lootData.Winner.name == currentTrade.tradeWith then
        sendMessage("  TRADE_SHOW() - wanting to give "..lootData.item.." to "..lootData.Winner.name..".", "DEBUG")
        --Should actually give the item away here....
        local bag, slot = findItemInBags(lootData.item)
        if bag == nil or slot == nil then
          sendMessage("  TRADE_SHOW() - I don't have the item to give away?", "DEBUG")
        else
          UseContainerItem(bag, slot)
          -- not auto accepting that is too scary
        end
      else
        sendMessage("  TRADE_SHOW() - Think I started it but the name i have doesn't match the person recieving the loot?", "DEBUG")
      end 
    else
      sendMessage("  TRADE_SHOW() - Think I started it but have no loot to give away?", "DEBUG")
    end
  else
    sendMessage("  TRADE_SHOW() probably not started by me", "DEBUG")
  end
end

---------
-- UTILITY FUNCTIONS
function sendMessage(message, channel)
  if message == nil then
    message = "";
  end;
  if type(message) == "table" then
    local tableMessage = message
    message = "["
    for index,value in pairs(tableMessage) do 
      if type(value) == "table" then
        local tableMessage2 = value
        message = message.."["
        for index2,value2 in pairs(tableMessage2) do 
          if type(value2) == "table" then
            message = message..tostring(index2).." : [another table], "
          else
            message = message..tostring(index2).." : "..value2..", "
          end
        end
        message = message.."]"
      else
        message = message..tostring(index).." : "..value..", "
      end
    end
    message = message.."]"
  end
  if channel == nil then
    channel = "SELF";
  end;
  if BLootAddonSaved.Config.debugMode_NoSpam == 1 then
    DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot ("..channel.."): "..message);
  else
    if channel=="RAID" then
      if GetNumGroupMembers() > 5 then
        SendChatMessage("Broken's Loot: "..message, "RAID");
      elseif GetNumGroupMembers() > 0 then
        SendChatMessage("Broken's Loot: "..message, "PARTY");
      else 
        DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot: "..message);
      end
    elseif (channel=="GUILD") then
        SendChatMessage("Broken's Loot: "..message, "GUILD") 
    elseif (channel=="SELF") then
      DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot: "..message);
    elseif (channel=="DEBUG") and (BLootAddonSaved.Config.debugMode == 1) then
      DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot (DEBUG): "..message, 1, 1, 0);
    end
  end
end

function isValidCharacter(characterName)
  if characterName == "Someone" then
    return true;
  end
  local i = getCharacterGroupIndex(characterName);
  sendMessage("isValidCharacter('"..characterName.."') is group at: "..i..".", "DEBUG");
  if i > 0 then
  	return true
  else
    sendMessage("isValidCharacter('"..characterName.."') is not in the group", "DEBUG");
    local numGroupMembers = GetNumGroupMembers()
    if numGroupMembers == 0 then
      local name = GetUnitName("player", true)
      if strlower(name) == strlower(characterName) then
        sendMessage("isValidCharacter('"..characterName.."') is the current character", "DEBUG");
        return true
      end
    end
  end
  return false
end

function getCharacterGroupIndex(characterName)
  sendMessage("getCharacterGroupIndex('"..characterName.."')", "DEBUG");
  local numGroupMembers = GetNumGroupMembers()
  for ci = 1, numGroupMembers do
    if strlower(GetRaidRosterInfo(ci)) == strlower(characterName) then
      sendMessage("getCharacterGroupIndex('"..characterName.."') = "..ci, "DEBUG");
      return ci;
    end
  end
  sendMessage("getCharacterGroupIndex('"..characterName.."') = not in group!", "DEBUG");
  return 0;
end

function isValidItem(itemLink)
  -- not checking it is in my bags initially so that I don't cause issues...
  --if findItemInBags(itemLink)
  --sendMessage("isValidItem - NYI");
  --debugItem(itemLink)
  return true;
end

function debugItem(itemLink)
  sendMessage("debugItem("..itemLink..")", "DEBUG");
  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink) 
  sendMessage("  itemType:"..itemType, "DEBUG");
  sendMessage("  itemSubType:"..itemSubType, "DEBUG");
  sendMessage("  itemEquipLoc"..itemEquipLoc, "DEBUG");
  getItemStats(itemLink);
end;

function getItemStats(itemLink)
  local stats = {};
  GetItemStats(itemLink, stats);
  sendMessage("getItemStats("..itemLink..")", "DEBUG")
  for key, value in pairs(stats) do 
    sendMessage("  "..key.." - " ..value, "DEBUG")
  end
  local agility, intellect, strength, stamina;
  agility = (stats["ITEM_MOD_AGILITY_SHORT"] or 0);
  intellect = (stats["ITEM_MOD_INTELLECT_SHORT"] or 0);
  strength = (stats["ITEM_MOD_STRENGTH_SHORT"] or 0);
  stamina = (stats["ITEM_MOD_STAMINA_SHORT"] or 0);
  sendMessage("  agi "..agility.." int "..intellect.." str "..strength.." stam "..stamina, "DEBUG");
  return agility, intellect, strength, stamina;
end

function generateLootId()
  BLootAddonSaved.lootId = BLootAddonSaved.lootId + 1;
  sendMessage("generateLootId() = "..BLootAddonSaved.lootId, "DEBUG")
  return BLootAddonSaved.lootId;
end

function getServerDate(andTime)
  local weekday, month, day, year = CalendarGetDate();
  if month < 10 then month = "0"..month; end;
  if day < 10 then day = "0"..day; end;
  if andTime then
    local hours,minutes = GetGameTime();
    if hours < 10 then hours = "0"..hours; end;
    if minutes < 10 then minutes = "0"..minutes; end;
    local currentDateTime = day.."/"..month.."/"..year.." "..hours..":"..minutes..":00";
    return currentDateTime;
  else
    local currentDate = day.."/"..month.."/"..year;
    return currentDate;
  end;
end

function moreThanAMinuteAgo(when)
-- NYI
  return true;
end

function findItemInBags(findItemId)
  sendMessage("findItemInBags("..findItemId..")", "DEBUG");
  for bag = 0, 5, 1 do 
    sendMessage("  Looking in bag ("..bag..")", "DEBUG");
    local numberOfSlots = GetContainerNumSlots(bag);
    sendMessage("  Bag "..bag.." has "..numberOfSlots.." slots.", "DEBUG");
    for slot = 1, numberOfSlots+1, 1 do
      itemLink = GetContainerItemLink(bag, slot);
      if not(itemLink == nil) then
        sendMessage("  Found at ("..bag..")("..slot..")"..itemLink, "DEBUG");
        if findItemId == itemLink then
          sendMessage("  Found at ("..bag..")("..slot..")", "DEBUG");
          return bag, slot;
        end
      end
    end
  end
  sendMessage("  Item not found", "DEBUG");
  return nil, nil;
end

function alreadyTrading()
  if enableTradeExtras then
    return currentTrade.openTrade;
  else
    return true;
  end
end

--[[ Function: extractItemLinkFromChatItemLink
     Purpose: gets an item link string from a chat item link string
     Note: don't ask questions, it works
  ]]
function extractItemLinkFromChatItemLink(message)
  local _, _, _, _itemLink = string.find(message, "(|H(.+)|h)");
  _, _, _itemLink = string.find(_itemLink, "(.-|h)");
  return _itemLink;
end
---------
-- STARTUP
SLASH_BROKENSLOOT1 = '/bloot';
SlashCmdList["BROKENSLOOT"] = handler;

function startup()
  localized = localizeStrings();
  if BLootAddonSaved == nil then
    BLootAddonSaved = {}
    BLootAddonSaved.lootId = 0;
  end
  if BLootAddonSaved.Config == nill then
    BLootAddonSaved.Config = {}
  end
  if BLootAddonSaved.Config.debugMode == nil then 
    BLootAddonSaved.Config.debugMode = 0; 
  elseif BLootAddonSaved.Config.debugMode == 1 then 
    sendMessage("Debug: on", "SELF") 
  end;
  if BLootAddonSaved.Config.debugMode_NoSpam == nil then 
    BLootAddonSaved.Config.debugMode_NoSpam = 0; 
  elseif BLootAddonSaved.Config.debugMode_NoSpam == 1 then 
    sendMessage("Debug No Spam: on", "SELF") 
  end;
  if BLootAddonSaved.Config.Countdown == nil then 
    BLootAddonSaved.Config.Countdown = 25; 
  elseif BLootAddonSaved.Config.debugMode == 1 then 
    sendMessage("Countdown: "..BLootAddonSaved.Config.Countdown, "SELF") 
  end;
  if BLootAddonSaved.Config.active == nil then 
    BLootAddonSaved.Config.active = 0; 
  elseif BLootAddonSaved.Config.active == 1 then 
    sendMessage("Active: on", "SELF") 
  end;
  sendMessage("Addon loaded...");
end

---------
-- ADDON VARIABLES
BrokensLoot = {}; -- holder for the panels
localized = {};

---------
-- EVENTS
local frame, events = CreateFrame("Frame"), {};

function events:CHAT_MSG_SYSTEM (...)
  --sendMessage("CHAT_MSG_SYSTEM", "DEBUG")
  local arg1 = select(1,...)
  if arg1 then
    --sendMessage("localized.rollResult: " .. localized.rollResult)
    local name, roll, minRoll, maxRoll = arg1:match(localized.rollResult)
    if name then
      -- we have found a roll - I think :)
      handleRoll(name, roll, minRoll, maxRoll);
    end
  end
end

function resetTrade()
  local t = {};
  t.openTrade = false;
  t.tradeWith = "Someone";
  t.startedByMe = false;
  t.wantToTradeWith = "";
  t.acceptPlayer = 0;
  t.acceptTarget = 1;
  t.playerItems = {};
  t.targetItems = {};
  t.close = 0;
  t.cancel = 0;
  return t;
end
function copyTrade(e)
  local t = {};
  t.openTrade = e.openTrade;
  t.tradeWith = e.tradeWith;
  t.startedByMe = e.startedByMe;
  t.wantToTradeWith = e.wantToTradeWith;
  t.acceptPlayer = e.acceptPlayer;
  t.acceptTarget = e.acceptTarget;
  t.playerItems = e.playerItems;
  t.targetItems = e.targetItems;
  t.close = e.close;
  t.cancel = e.cancel;
  return t;
end

currentTrade = resetTrade();

if enableTradeExtras then
  function events:TRADE_SHOW(...)
    if BLootAddonSaved.Config.active then
      sendMessage("TRADE_SHOW()", "DEBUG")
      currentTrade.openTrade = true;
      handleTradeStarted()
    end
  end

  function events:TRADE_ACCEPT_UPDATE(...)
    if BLootAddonSaved.Config.active then
      local arg1, arg2  = select(1,...)
      currentTrade.acceptPlayer = arg1
      currentTrade.acceptTarget = arg2
    end
  end
  -- don't think this event exists any more
  function events:TRADE_REQUEST(...)
    if BLootAddonSaved.Config.active then
      sendMessage("TRADE_REQUEST()", "DEBUG")
      local arg1  = select(1,...)
      currentTrade.tradeWith = arg1;
    end
  end

  -- player cancelled - CLOSE CLOSE CANCEL  ??
  -- target cancelled - CANCEL CLOSE CLOSE  ??

  -- Fired when a trade attempt is cancelled. Fired after TRADE_CLOSE when aborted by player, before TRADE_CLOSE when done by target.
  -- Upon a trade being cancelled (as in, either part clicking the cancel button), TRADE_CLOSE is fired twice, and then TRADE_REQUEST_CANCEL once.
  function events:TRADE_REQUEST_CANCEL(...)
    if BLootAddonSaved.Config.active then
      sendMessage("TRADE_REQUEST_CANCEL()", "DEBUG")
      currentTrade.cancel = currentTrade.cancel + 1;
      if currentTrade.inProgress == true then
      else
        if currentTrade.inProgress == true then
        else
          currentTrade.inProgress = true;
          C_Timer.After(2, handleTradeComplete)
        end
      end
    end
  end

  function events:TRADE_CLOSED(...)
    if BLootAddonSaved.Config.active then
      -- dirty hack in here for now - you have to target the person before hitting accept so i know who it is? just a test
      --currentTrade.tradeWith = UnitName("target")
      currentTrade.close = currentTrade.close + 1;
      sendMessage("TRADE_CLOSED()", "DEBUG")
      if currentTrade.inProgress == true then
      else
        if currentTrade.inProgress == true then
        else
          currentTrade.inProgress = true;
          C_Timer.After(2, handleTradeComplete)
        end
      end
    end
  end

  function events:TRADE_TARGET_ITEM_CHANGED(...)
    if BLootAddonSaved.Config.active then
      local arg1 = select(1,...)
      sendMessage("TRADE_PLAYER_ITEM_CHANGED("..arg1..")", "DEBUG")
      currentTrade.playerItems = {};
      currentTrade.targetItems = {};
      for tradeSlotId = 1, 7 do
        playerSideItemLink = GetTradePlayerItemLink(tradeSlotId);
        if not(playerSideItemLink == nil) then
          sendMessage("  playerSideItemLink: "..playerSideItemLink..".", "DEBUG")
          tinsert(currentTrade.playerItems, playerSideItemLink);
        end
        targetSideItemLink = GetTradeTargetItemLink(tradeSlotId);
        if not(targetSideItemLink == nil) then
          sendMessage("  targetSideItemLink"..targetSideItemLink..".", "DEBUG")
          tinsert(currentTrade.targetItems, targetSideItemLink);
        end
      end
    end
  end
  function events:TRADE_PLAYER_ITEM_CHANGED(...)
    if BLootAddonSaved.Config.active then
      local arg1 = select(1,...)
      sendMessage("TRADE_PLAYER_ITEM_CHANGED("..arg1..")", "DEBUG")
      currentTrade.playerItems = {};
      currentTrade.targetItems = {};
      for tradeSlotId = 1, 7 do
        playerSideItemLink = GetTradePlayerItemLink(tradeSlotId);
        if not(playerSideItemLink == nil) then
          sendMessage("  playerSideItemLink: "..playerSideItemLink..".", "DEBUG")
          tinsert(currentTrade.playerItems, playerSideItemLink);
        end
        targetSideItemLink = GetTradeTargetItemLink(tradeSlotId);
        if not(targetSideItemLink == nil) then
          sendMessage("  targetSideItemLink"..targetSideItemLink..".", "DEBUG")
          tinsert(currentTrade.targetItems, targetSideItemLink);
        end
      end
    end
  end
end

function events:VARIABLES_LOADED (...)
  startup();
end

frame:SetScript("OnEvent", function(self, event, ...)
  events[event](self, ...); -- call one of the functions above
end);

for k, v in pairs(events) do
  frame:RegisterEvent(k); -- Register all events for which handlers have been defined
end

---------
-- LOCALISATION
function localizeStrings()
  local loc = {};
  loc.rollResult = RANDOM_ROLL_RESULT;
  --sendMessage("START:" .. loc.rollResult)
  --sendMessage("WANT :" .. "^(.+) rolls (%d+) %((%d+)%-(%d+)%)$");
  -- ("^(.+) rolls (%d+) %((%d+)%-(%d+)%)$")
  -- ("%s rolls %d (%d-%d)")
  loc.rollResult = gsub(loc.rollResult, "%%s", "^(.+)", 1)
  --sendMessage("GOT1 :" .. loc.rollResult)
  loc.rollResult = gsub(loc.rollResult, "%%d", "(%%d+)", 1)
  --sendMessage("GOT2 :" .. loc.rollResult)
  loc.rollResult = gsub(loc.rollResult, "%(%%d%-%%d%)", "%%((%%d+)%%-(%%d+)%%)$", 1)
  --sendMessage("GOT3 :" .. loc.rollResult)
  --sendMessage("WANT :" .. "^(.+) rolls (%d+) %((%d+)%-(%d+)%)$");
  return loc;
end